@@ -3,7 +3,9 @@
     call the call-back-method.
 """
 
-import os, json
+import os
+import json
+import csv
 import threading
 import logging
 from flask import current_app as app
@@ -42,14 +44,14 @@ def read_json_file(path):
 
 
 def _delete_folder(path):
-    """ Internal method which won't trigger the callback """
+    """Internal method which won't trigger the callback"""
     with fslock:
         if os.path.exists(path):
             shutil.rmtree(path)
 
 
 def _write_json_file(content, path, lock=None):
-    """ Internal method which won't trigger the callback """
+    """Internal method which won't trigger the callback"""
     if lock is None:
         lock = fslock
     with lock:
@@ -75,7 +77,7 @@ def _write_json_file(content, path, lock=None):
             if os.path.isfile(path):
                 os.remove(path)
             os.rename(bkp, path)
-            raise RuntimeError(f"Failed to write to file {path}")
+            logger.error(f"Failed to write to file {path}")
 
 
 def write_json_file(content, path, lock=None):
@@ -83,14 +85,23 @@ def write_json_file(content, path, lock=None):
     storage_callback()
 
 
-def delete_json_file(path):
-    if os.path.exists(path):
-        os.remove(path)
-    storage_callback()
+def delete_files(paths):
+    """deletes multiple files and calls storage callback once"""
+    need_callback = False
+    for path in paths:
+        if os.path.exists(path):
+            os.remove(path)
+            need_callback = True
+    if need_callback:
+        storage_callback()
+
+
+def delete_file(path):
+    delete_files([path])
 
 
 def write_devices(devices_json):
-    """ interpret a json as a list of devices and write them in the devices subfolder inside the specter-folder """
+    """interpret a json as a list of devices and write them in the devices subfolder inside the specter-folder"""
     for device_json in devices_json:
         _write_json_file(
             device_json,
@@ -128,6 +139,35 @@ def delete_folders(paths):
     storage_callback()
 
 
+def _write_csv(fname, objs, cls=dict):
+    columns = []
+    # if it's a custom class
+    if hasattr(cls, "columns"):
+        columns = cls.columns
+    # if it's just a dict
+    elif len(objs) > 0:
+        columns = objs[0].keys()
+
+    with fslock:
+        with open(fname, mode="w") as csv_file:
+            writer = csv.DictWriter(csv_file, fieldnames=columns)
+            writer.writeheader()
+            for obj in objs:
+                writer.writerow(obj)
+
+
+def write_csv(fname, objs, cls=dict):
+    _write_csv(fname, objs, cls=dict)
+    storage_callback()
+
+
+def read_csv(fname, cls=dict, *args):
+    with fslock:
+        with open(fname, mode="r") as csv_file:
+            csv_reader = csv.DictReader(csv_file)
+            return [cls(*args, **row) for row in csv_reader]
+
+
 def storage_callback():
     if os.getenv("SPECTER_PERSISTENCE_CALLBACK"):
         result = run_shell(os.getenv("SPECTER_PERSISTENCE_CALLBACK").split(" "))
