@@ -25,7 +25,7 @@ auth_endpoint = Blueprint("auth_endpoint", __name__)
 @auth_endpoint.route("/login", methods=["GET", "POST"])
 @app.csrf.exempt
 def login():
-    """ login """
+    """login"""
     if request.method == "POST":
         rate_limit()
         auth = app.specter.config["auth"]
@@ -97,7 +97,7 @@ def login():
 
 @auth_endpoint.route("/register", methods=["GET", "POST"])
 def register():
-    """ register """
+    """register"""
     if request.method == "POST":
         rate_limit()
         username = request.form["username"]
@@ -116,7 +116,7 @@ def register():
                 "error",
             )
             return redirect("register?otp={}".format(otp))
-        if app.specter.validate_new_user_otp(otp):
+        if app.specter.otp_manager.validate_new_user_otp(otp):
             user_id = alias(username)
             i = 1
             while app.specter.user_manager.get_user(user_id):
@@ -125,14 +125,14 @@ def register():
             if app.specter.user_manager.get_user_by_username(username):
                 flash("Username is already taken, please choose another one", "error")
                 return redirect("register?otp={}".format(otp))
-            app.specter.remove_new_user_otp(otp)
+            app.specter.otp_manager.remove_new_user_otp(otp)
             config = {
                 "explorers": {"main": "", "test": "", "regtest": "", "signet": ""},
                 "hwi_bridge_url": "/hwi/api/",
             }
             password_hash = hash_password(password)
             user = User(user_id, username, password_hash, config, app.specter)
-            app.specter.add_user(user)
+            app.specter.user_manager.add_user(user)
             flash(
                 "You have registered successfully, \
 please login with your new account to start using Specter"
