@@ -1,4 +1,4 @@
-import copy, random
+import copy, random, json
 
 from flask import (
     Flask,
@@ -15,6 +15,8 @@ from flask import current_app as app
 from mnemonic import Mnemonic
 from ..helpers import is_testnet, generate_mnemonic
 from ..key import Key
+from ..device_manager import get_device_class
+from ..devices.bitcoin_core import BitcoinCore
 from ..wallet_manager import purposes
 from ..specter_error import handle_exception
 
@@ -25,138 +27,195 @@ devices_endpoint = Blueprint("devices_endpoint", __name__)
 
 
 ################## New device #######################
-
-# New device
-@devices_endpoint.route("/new_device/", methods=["GET", "POST"])
+# New device type
+@devices_endpoint.route("/new_device_type/", methods=["GET", "POST"])
 @login_required
-def new_device():
+def new_device_type():
+    return render_template(
+        "device/new_device/new_device_type.jinja",
+        specter=app.specter,
+        rand=rand,
+    )
+
+
+@devices_endpoint.route("/new_device_keys/<device_type>", methods=["GET", "POST"])
+@login_required
+def new_device_keys(device_type):
     err = None
-    strength = 128
-    mnemonic = generate_mnemonic(strength=strength)
+    mnemonic = ""
+    passphrase = ""
+    file_password = ""
+    range_start = 0
+    range_end = 1000
+    existing_device = None
     if request.method == "POST":
-        if request.form.get("existing_device"):
-            device = app.specter.device_manager.get_by_alias(
-                request.form.get("existing_device")
-            )
-            device_type = device.device_type
+        mnemonic = request.form.get("mnemonic", "")
+        passphrase = request.form.get("passphrase", "")
+        file_password = request.form.get("file_password", "")
+        range_start = int(request.form.get("range_start", "0"))
+        range_end = int(request.form.get("range_end", "1000"))
+        existing_device = request.form.get("existing_device", None)
+        if existing_device:
+            device = app.specter.device_manager.get_by_alias(existing_device)
         else:
-            device_type = request.form.get("devices")
             device_name = request.form.get("device_name", "")
             if not device_name:
                 err = "Device name must not be empty"
             elif device_name in app.specter.device_manager.devices_names:
                 err = "Device with this name already exists"
         xpubs_rows_count = int(request.form["xpubs_rows_count"]) + 1
-        if device_type != "bitcoincore":
-            keys = []
-            for i in range(0, xpubs_rows_count):
-                purpose = request.form.get(
-                    "xpubs-table-row-{}-purpose".format(i), "Custom"
-                )
-                xpub = request.form.get("xpubs-table-row-{}-xpub-hidden".format(i), "-")
-                if xpub != "-":
-                    try:
-                        keys.append(Key.parse_xpub(xpub, purpose=purpose))
-                    except:
-                        err = "Failed to parse these xpubs:\n" + "\n".join(xpub)
-                        break
-            if not keys and not err:
-                err = "xpubs name must not be empty"
-            if err is None:
-                if request.form.get("existing_device"):
-                    device.add_keys(keys)
-                    flash("{} keys were added successfully".format(len(keys)))
-                    return redirect(
-                        url_for("devices_endpoint.device", device_alias=device.alias)
+        keys = []
+        paths = []
+        keys_purposes = []
+        for i in range(0, xpubs_rows_count):
+            purpose = request.form.get("xpubs-table-row-{}-purpose".format(i), "Custom")
+            xpub = request.form.get("xpubs-table-row-{}-xpub-hidden".format(i), "-")
+            path = request.form.get(
+                "xpubs-table-row-{}-derivation-hidden".format(i), ""
+            )
+            if path != "":
+                paths.append(path)
+                keys_purposes.append(purpose)
+            if xpub != "-":
+                try:
+                    keys.append(Key.parse_xpub(xpub, purpose=purpose))
+                except:
+                    err = "Failed to parse these xpubs:\n" + "\n".join(xpub)
+                    break
+        if not keys and not err:
+            if device_type == "bitcoincore":
+                if not paths:
+                    err = "No paths were specified, please provide at lease one."
+                if err is None:
+                    if existing_device:
+                        device.add_hot_wallet_keys(
+                            mnemonic,
+                            passphrase,
+                            paths,
+                            file_password,
+                            app.specter.wallet_manager,
+                            is_testnet(app.specter.chain),
+                            keys_range=[range_start, range_end],
+                            keys_purposes=keys_purposes,
+                        )
+                        flash("{} keys were added successfully".format(len(paths)))
+                        return redirect(
+                            url_for(
+                                "devices_endpoint.device", device_alias=device.alias
+                            )
+                        )
+                    device = app.specter.device_manager.add_device(
+                        name=device_name, device_type=device_type, keys=[]
                     )
-                device = app.specter.device_manager.add_device(
-                    name=device_name, device_type=device_type, keys=keys
-                )
-                flash("{} was added successfully!".format(device_name))
+                    try:
+                        device.setup_device(file_password, app.specter.wallet_manager)
+                        device.add_hot_wallet_keys(
+                            mnemonic,
+                            passphrase,
+                            paths,
+                            file_password,
+                            app.specter.wallet_manager,
+                            is_testnet(app.specter.chain),
+                            keys_range=[range_start, range_end],
+                            keys_purposes=keys_purposes,
+                        )
+                        flash("{} was added successfully!".format(device_name))
+                        return redirect(
+                            url_for(
+                                "devices_endpoint.device", device_alias=device.alias
+                            )
+                            + "?newdevice=true"
+                        )
+                    except Exception as e:
+                        handle_exception(e)
+                        flash(f"Failed to setup hot wallet. Error: {e}", "error")
+                        app.specter.device_manager.remove_device(
+                            device,
+                            app.specter.wallet_manager,
+                            bitcoin_datadir=app.specter.bitcoin_datadir,
+                            chain=app.specter.chain,
+                        )
+            else:
+                err = "xpubs list must not be empty"
+        elif not err:
+            if existing_device:
+                device.add_keys(keys)
+                flash("{} keys were added successfully".format(len(keys)))
                 return redirect(
                     url_for("devices_endpoint.device", device_alias=device.alias)
-                    + "?newdevice=true"
                 )
-            else:
-                flash(err, "error")
-        else:
-            if len(request.form["mnemonic"].split(" ")) not in [12, 15, 18, 21, 24]:
-                err = "Invalid mnemonic entered: Must contain either: 12, 15, 18, 21, or 24 words."
-            mnemo = Mnemonic("english")
-            if not mnemo.check(request.form["mnemonic"]):
-                err = "Invalid mnemonic entered."
-            range_start = int(request.form["range_start"])
-            range_end = int(request.form["range_end"])
-            if range_start > range_end:
-                err = "Invalid address range selected."
-            mnemonic = request.form["mnemonic"]
-            paths = []
-            keys_purposes = []
-            for i in range(0, xpubs_rows_count):
-                purpose = request.form.get(
-                    "xpubs-table-row-{}-purpose".format(i), "Custom"
-                )
-                path = request.form.get(
-                    "xpubs-table-row-{}-derivation-hidden".format(i), ""
-                )
-                if path != "":
-                    paths.append(path)
-                    keys_purposes.append(purpose)
-            if not paths:
-                err = "No paths were specified, please provide at lease one."
-            if err is None:
-                passphrase = request.form["passphrase"]
-                file_password = request.form["file_password"]
-                if request.form.get("existing_device"):
-                    device.add_hot_wallet_keys(
-                        mnemonic,
-                        passphrase,
-                        paths,
-                        file_password,
-                        app.specter.wallet_manager,
-                        is_testnet(app.specter.chain),
-                        keys_range=[range_start, range_end],
-                        keys_purposes=keys_purposes,
-                    )
-                    flash("{} keys were added successfully".format(len(paths)))
-                    return redirect(
-                        url_for("devices_endpoint.device", device_alias=device.alias)
-                    )
-                device = app.specter.device_manager.add_device(
-                    name=device_name, device_type=device_type, keys=[]
-                )
-                try:
-                    device.setup_device(file_password, app.specter.wallet_manager)
-                    device.add_hot_wallet_keys(
-                        mnemonic,
-                        passphrase,
-                        paths,
-                        file_password,
-                        app.specter.wallet_manager,
-                        is_testnet(app.specter.chain),
-                        keys_range=[range_start, range_end],
-                        keys_purposes=keys_purposes,
-                    )
-                    flash("{} was added successfully!".format(device_name))
-                    return redirect(
-                        url_for("devices_endpoint.device", device_alias=device.alias)
-                        + "?newdevice=true"
-                    )
-                except Exception as e:
-                    handle_exception(e)
-                    flash(f"Failed to setup hot wallet. Error: {e}", "error")
-                    app.specter.device_manager.remove_device(
-                        device,
-                        app.specter.wallet_manager,
-                        bitcoin_datadir=app.specter.bitcoin_datadir,
-                        chain=app.specter.chain,
-                    )
-            else:
-                flash(err, "error")
+            device = app.specter.device_manager.add_device(
+                name=device_name, device_type=device_type, keys=keys
+            )
+            flash("{} was added successfully!".format(device_name))
+            return redirect(
+                url_for("devices_endpoint.device", device_alias=device.alias)
+                + "?newdevice=true"
+            )
+
     return render_template(
-        "device/new_device.jinja",
+        "device/new_device/new_device_keys.jinja",
+        device_class=get_device_class(device_type),
         mnemonic=mnemonic,
+        passphrase=passphrase,
+        file_password=file_password,
+        range_start=range_start,
+        range_end=range_end,
+        existing_device=app.specter.device_manager.get_by_alias(existing_device)
+        if existing_device
+        else None,
+        error=err,
+        specter=app.specter,
+        rand=rand,
+    )
+
+
+@devices_endpoint.route("/new_device_mnemonic/", methods=["GET", "POST"])
+@login_required
+def new_device_mnemonic():
+    err = None
+    strength = 128
+    mnemonic = generate_mnemonic(strength=strength)
+    existing_device = None
+    if request.method == "POST":
+        if len(request.form["mnemonic"].split(" ")) not in [12, 15, 18, 21, 24]:
+            err = "Invalid mnemonic entered: Must contain either: 12, 15, 18, 21, or 24 words."
+        mnemo = Mnemonic("english")
+        if not mnemo.check(request.form["mnemonic"]):
+            err = "Invalid mnemonic entered."
+        range_start = int(request.form["range_start"])
+        range_end = int(request.form["range_end"])
+        if range_start > range_end:
+            err = "Invalid address range selected."
+        mnemonic = request.form["mnemonic"]
+        passphrase = request.form["passphrase"]
+        file_password = request.form["file_password"]
+        existing_device = request.form.get("existing_device", None)
+        print("file_password")
+        print(file_password)
+        if existing_device:
+            existing_device = app.specter.device_manager.get_by_alias(existing_device)
+        if not err:
+            return render_template(
+                "device/new_device/new_device_keys.jinja",
+                mnemonic=mnemonic,
+                passphrase=passphrase,
+                file_password=file_password,
+                range_start=range_start,
+                range_end=range_end,
+                device_class=BitcoinCore,
+                existing_device=existing_device,
+                error=err,
+                specter=app.specter,
+                rand=rand,
+            )
+
+    return render_template(
+        "device/new_device/new_device_mnemonic.jinja",
         strength=strength,
+        mnemonic=mnemonic,
+        existing_device=existing_device,
+        error=err,
         specter=app.specter,
         rand=rand,
     )
@@ -312,62 +371,26 @@ def device(device_alias):
         elif action == "add_keys":
             strength = 128
             mnemonic = generate_mnemonic(strength=strength)
-            return render_template(
-                "device/new_device.jinja",
-                mnemonic=mnemonic,
-                strength=strength,
-                existing_device=device,
-                device_alias=device_alias,
-                specter=app.specter,
-                rand=rand,
-            )
-        elif action == "morekeys":
             if device.hot_wallet:
-                if len(request.form["mnemonic"].split(" ")) not in [12, 15, 18, 21, 24]:
-                    err = "Invalid mnemonic entered: Must contain either: 12, 15, 18, 21, or 24 words."
-                mnemo = Mnemonic("english")
-                if not mnemo.check(request.form["mnemonic"]):
-                    err = "Invalid mnemonic entered."
-                range_start = int(request.form["range_start"])
-                range_end = int(request.form["range_end"])
-                if range_start > range_end:
-                    err = "Invalid address range selected."
-                if err is None:
-                    mnemonic = request.form["mnemonic"]
-                    paths = [
-                        l.strip()
-                        for l in request.form["derivation_paths"].split("\n")
-                        if len(l) > 0
-                    ]
-                    passphrase = request.form["passphrase"]
-                    file_password = request.form["file_password"]
-                    device.add_hot_wallet_keys(
-                        mnemonic,
-                        passphrase,
-                        paths,
-                        file_password,
-                        app.specter.wallet_manager,
-                        is_testnet(app.specter.chain),
-                        keys_range=[range_start, range_end],
-                    )
+                return render_template(
+                    "device/new_device/new_device_mnemonic.jinja",
+                    mnemonic=mnemonic,
+                    strength=strength,
+                    existing_device=device,
+                    device_alias=device_alias,
+                    device_class=get_device_class(device.device_type),
+                    specter=app.specter,
+                    rand=rand,
+                )
             else:
-                # refactor to fn
-                xpubs = request.form["xpubs"]
-                keys, failed = Key.parse_xpubs(xpubs)
-                err = None
-                if len(failed) > 0:
-                    err = "Failed to parse these xpubs:\n" + "\n".join(failed)
-                    return render_template(
-                        "device/new_device_manual.jinja",
-                        device=device,
-                        device_alias=device_alias,
-                        xpubs=xpubs,
-                        error=err,
-                        specter=app.specter,
-                        rand=rand,
-                    )
-                if err is None:
-                    device.add_keys(keys)
+                return render_template(
+                    "device/new_device/new_device_keys.jinja",
+                    existing_device=device,
+                    device_alias=device_alias,
+                    device_class=get_device_class(device.device_type),
+                    specter=app.specter,
+                    rand=rand,
+                )
         elif action == "settype":
             device_type = request.form["device_type"]
             device.set_type(device_type)
