@@ -1,24 +1,25 @@
-import ast, csv, json, os, time, base64, random, requests, logging
-from io import StringIO
-from werkzeug.wrappers import Response
+import ast
+import base64
+import csv
+import json
+import logging
+import os
+import random
+import time
+from binascii import b2a_base64
 from datetime import datetime
-from numbers import Number
+from functools import wraps
+from io import StringIO
 from math import isnan
-from ..util.tx import decoderawtransaction
-from ..util.price_providers import get_price_at
+from numbers import Number
+
+import requests
+from flask import Blueprint, Flask
+from flask import current_app as app
+from flask import flash, jsonify, redirect, render_template, request, url_for
+from flask_login import current_user, login_required
+from werkzeug.wrappers import Response
 
-from flask import (
-    Flask,
-    Blueprint,
-    render_template,
-    request,
-    redirect,
-    url_for,
-    jsonify,
-    flash,
-)
-from flask_login import login_required, current_user
-from ..util.descriptor import AddChecksum, Descriptor
 from ..helpers import (
     bcur2base64,
     get_devices_with_keys_by_type,
@@ -26,16 +27,17 @@ from ..helpers import (
     is_testnet,
     parse_wallet_data_import,
 )
-from ..persistence import delete_file
 from ..key import Key
-from ..specter import Specter
-from ..specter_error import SpecterError
-from ..wallet_manager import purposes
+from ..persistence import delete_file
 from ..rpc import RpcError
-from binascii import b2a_base64
+from ..specter import Specter
+from ..specter_error import SpecterError, handle_exception
 from ..util.base43 import b43_decode
-
-from flask import current_app as app
+from ..util.descriptor import AddChecksum, Descriptor
+from ..util.fee_estimation import get_fees
+from ..util.price_providers import get_price_at
+from ..util.tx import decoderawtransaction
+from ..wallet_manager import purposes
 
 rand = random.randint(0, 1e32)  # to force style refresh
 
@@ -43,6 +45,27 @@ rand = random.randint(0, 1e32)  # to force style refresh
 wallets_endpoint = Blueprint("wallets_endpoint", __name__)
 
 
+def handle_wallet_error(func_name, error):
+    flash(f"SpecterError while {func_name}: {error}", "error")
+    app.logger.error(f"SpecterError while {func_name}: {error}")
+    app.specter.wallet_manager.update()
+    return redirect(url_for("about"))
+
+
+def check_wallet(func):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        """checks the wallet for healthyness A wrapper function"""
+        if kwargs["wallet_alias"]:
+            wallet_alias = kwargs["wallet_alias"]
+            print("--------------------checking wallet")
+            wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
+            wallet.get_info()
+        return func(*args, **kwargs)
+
+    return wrapper
+
+
 ################## Wallet overview #######################
 
 
@@ -50,9 +73,10 @@ wallets_endpoint = Blueprint("wallets_endpoint", __name__)
 @login_required
 def wallets_overview():
     app.specter.check_blockheight()
-    for wallet in app.specter.wallet_manager.wallets.values():
+    for wallet in list(app.specter.wallet_manager.wallets.values()):
         wallet.get_balance()
         wallet.check_utxo()
+
     return render_template(
         "wallet/wallets_overview.jinja",
         specter=app.specter,
@@ -80,6 +104,7 @@ def failed_wallets():
                 delete_file(fullpath.replace(".json", "_txs.csv"))
                 app.specter.wallet_manager.update()
             except Exception as e:
+
                 flash(f"Failed to delete failed wallet: {str(e)}", "error")
     return redirect("/")
 
@@ -118,14 +143,16 @@ def new_wallet(wallet_type):
         action = request.form["action"]
         if action == "importwallet":
             try:
-                wallet_data = json.loads(request.form["wallet_data"].replace("'", "h"))
+                wallet_data = json.loads(
+                    request.form["wallet_data"].replace("\\'", "").replace("'", "h")
+                )
                 (
                     wallet_name,
                     recv_descriptor,
                     cosigners_types,
                 ) = parse_wallet_data_import(wallet_data)
-            except Exception:
-                flash("Unsupported wallet import format", "error")
+            except Exception as e:
+                flash(f"Unsupported wallet import format:{e}", "error")
                 return redirect(url_for("wallets_endpoint.new_wallet_type"))
             # get min of the two
             # if the node is still syncing
@@ -369,8 +396,8 @@ def new_wallet(wallet_type):
                 wallet = app.specter.wallet_manager.create_wallet(
                     wallet_name, sigs_required, address_type, keys, cosigners
                 )
-            except Exception:
-                err = "Failed to create wallet..."
+            except Exception as e:
+                err = f"Failed to create wallet. Error: {e}"
                 return render_template(
                     "wallet/new_wallet/new_wallet_keys.jinja",
                     cosigners=cosigners,
@@ -391,7 +418,12 @@ def new_wallet(wallet_type):
                 if "utxo" in request.form.get("full_rescan_option"):
                     explorer = None
                     if "use_explorer" in request.form:
-                        explorer = request.form["explorer_url"]
+                        if request.form["explorer"] == "CUSTOM":
+                            explorer = request.form["custom_explorer"]
+                        else:
+                            explorer = app.config["EXPLORERS_LIST"][
+                                request.form["explorer"]
+                            ]["url"]
                     wallet.rescanutxo(
                         explorer,
                         app.specter.requests_session(explorer),
@@ -446,11 +478,7 @@ def new_wallet(wallet_type):
 @wallets_endpoint.route("/wallet/<wallet_alias>/")
 @login_required
 def wallet(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     if wallet.fullbalance > 0:
         return redirect(url_for("wallets_endpoint.history", wallet_alias=wallet_alias))
     else:
@@ -461,12 +489,7 @@ def wallet(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/history/", methods=["GET", "POST"])
 @login_required
 def history(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet_tx: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
-
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     tx_list_type = "txlist"
 
     if request.method == "POST":
@@ -474,6 +497,12 @@ def history(wallet_alias):
         if action == "freezeutxo":
             wallet.toggle_freeze_utxo(request.form.getlist("selected_utxo"))
             tx_list_type = "utxo"
+        elif action == "abandon_tx":
+            txid = request.form["txid"]
+            try:
+                wallet.abandontransaction(txid)
+            except SpecterError as e:
+                flash(str(e), "error")
 
     # update balances in the wallet
     app.specter.check_blockheight()
@@ -495,12 +524,9 @@ def history(wallet_alias):
 
 @wallets_endpoint.route("/wallet/<wallet_alias>/receive/", methods=["GET", "POST"])
 @login_required
+@check_wallet
 def receive(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet_receive: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     if request.method == "POST":
         action = request.form["action"]
         if action == "newaddress":
@@ -526,11 +552,7 @@ def receive(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/send")
 @login_required
 def send(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet_send: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     if len(wallet.pending_psbts) > 0:
         return redirect(
             url_for("wallets_endpoint.send_pending", wallet_alias=wallet_alias)
@@ -542,11 +564,7 @@ def send(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/send/new", methods=["GET", "POST"])
 @login_required
 def send_new(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet_send: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     # update balances in the wallet
     wallet.get_balance()
     # update utxo list for coin selection
@@ -562,12 +580,19 @@ def send_new(wallet_alias):
     subtract = False
     subtract_from = 1
     fee_options = "dynamic"
-    fee_rate = 0.0
-    fee_rate_blocks = 6
     rbf = True
+    rbf_utxo = []
+    rbf_tx_id = ""
     selected_coins = request.form.getlist("coinselect")
+    fee_estimation_data = get_fees(app.specter, app.config)
+    if fee_estimation_data.get("failed", None):
+        flash("Failed to fetch fee estimations, please use the manual fee calculation")
+
+    fee_rate = fee_estimation_data["hourFee"]
+
     if request.method == "POST":
         action = request.form.get("action")
+        rbf_tx_id = request.form.get("rbf_tx_id", "")
         if action == "createpsbt":
             i = 0
             addresses = []
@@ -612,7 +637,6 @@ def send_new(wallet_alias):
             if fee_options:
                 if "dynamic" in fee_options:
                     fee_rate = float(request.form.get("fee_rate_dynamic"))
-                    fee_rate_blocks = int(request.form.get("fee_rate_blocks"))
                 else:
                     if request.form.get("fee_rate"):
                         fee_rate = float(request.form.get("fee_rate"))
@@ -629,6 +653,7 @@ def send_new(wallet_alias):
                     rbf=rbf,
                     selected_coins=selected_coins,
                     readonly="estimate_fee" in request.form,
+                    rbf_edit_mode=(rbf_tx_id != ""),
                 )
                 if psbt is None:
                     err = "Probably you don't have enough funds, or something else..."
@@ -660,7 +685,7 @@ def send_new(wallet_alias):
             try:
                 rbf_tx_id = request.form["rbf_tx_id"]
                 rbf_fee_rate = float(request.form["rbf_fee_rate"])
-                psbt = wallet.send_rbf_tx(rbf_tx_id, rbf_fee_rate)
+                psbt = wallet.bumpfee(rbf_tx_id, rbf_fee_rate)
                 return render_template(
                     "wallet/send/sign/wallet_send_sign_psbt.jinja",
                     psbt=psbt,
@@ -672,6 +697,20 @@ def send_new(wallet_alias):
                 )
             except Exception as e:
                 flash("Failed to perform RBF. Error: %s" % e, "error")
+        elif action == "rbf_edit":
+            try:
+                decoded_tx = wallet.decode_tx(rbf_tx_id)
+                addresses = decoded_tx["addresses"]
+                amounts = decoded_tx["amounts"]
+                selected_coins = [
+                    f"{utxo['txid']}, {utxo['vout']}"
+                    for utxo in decoded_tx["used_utxo"]
+                ]
+                fee_rate = float(request.form["rbf_fee_rate"])
+                fee_options = "manual"
+                rbf = True
+            except Exception as e:
+                flash("Failed to perform RBF. Error: %s" % e, "error")
         elif action == "signhotwallet":
             passphrase = request.form["passphrase"]
             psbt = ast.literal_eval(request.form["psbt"])
@@ -708,14 +747,22 @@ def send_new(wallet_alias):
                 specter=app.specter,
                 rand=rand,
             )
+
+    if rbf_tx_id:
+        try:
+            rbf_utxo = wallet.get_rbf_utxo(rbf_tx_id)
+        except Exception as e:
+            flash("Failed to get RBF coins. Error: %s" % e, "error")
+
     show_advanced_settings = (
         ui_option != "ui"
         or subtract
         or fee_options != "dynamic"
-        or fee_rate_blocks != 6
+        or fee_estimation_data["hourFee"] != fee_rate
         or not rbf
         or selected_coins
     )
+
     return render_template(
         "wallet/send/new/wallet_send.jinja",
         psbt=psbt,
@@ -726,10 +773,13 @@ def send_new(wallet_alias):
         subtract_from=subtract_from,
         fee_options=fee_options,
         fee_rate=fee_rate,
-        fee_rate_blocks=fee_rate_blocks,
         rbf=rbf,
         selected_coins=selected_coins,
         show_advanced_settings=show_advanced_settings,
+        rbf_utxo=rbf_utxo,
+        rbf_tx_id=rbf_tx_id,
+        fee_estimation=fee_rate,
+        fee_estimation_data=fee_estimation_data,
         wallet_alias=wallet_alias,
         wallet=wallet,
         specter=app.specter,
@@ -741,11 +791,7 @@ def send_new(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/send/pending/", methods=["GET", "POST"])
 @login_required
 def send_pending(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet_sendpending: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     if request.method == "POST":
         action = request.form["action"]
         if action == "deletepsbt":
@@ -785,11 +831,7 @@ def send_pending(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/send/import", methods=["GET", "POST"])
 @login_required
 def import_psbt(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet_send: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     if request.method == "POST":
         action = request.form["action"]
         if action == "importpsbt":
@@ -829,12 +871,7 @@ def addresses(wallet_alias):
     """Show informations about cached addresses (wallet._addresses) of the <wallet_alias>.
     It updates balances in the wallet before renderization in order to show updated UTXO and
     balance of each address."""
-
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet_send: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
 
     # update balances in the wallet
     app.specter.check_blockheight()
@@ -856,12 +893,8 @@ def addresses(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/settings/", methods=["GET", "POST"])
 @login_required
 def settings(wallet_alias):
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     error = None
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while wallet_receive: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
     if request.method == "POST":
         action = request.form["action"]
         if action == "rescanblockchain":
@@ -885,7 +918,13 @@ def settings(wallet_alias):
         elif action == "rescanutxo":
             explorer = None
             if "use_explorer" in request.form:
-                explorer = request.form["explorer_url"]
+                if request.form["explorer"] == "CUSTOM":
+                    explorer = request.form["custom_explorer"]
+                else:
+                    explorer = app.config["EXPLORERS_LIST"][request.form["explorer"]][
+                        "url"
+                    ]
+
             wallet.rescanutxo(
                 explorer, app.specter.requests_session(explorer), app.specter.only_tor
             )
@@ -919,25 +958,15 @@ def settings(wallet_alias):
             else:
                 app.specter.wallet_manager.rename_wallet(wallet, wallet_name)
 
-        return render_template(
-            "wallet/settings/wallet_settings.jinja",
-            purposes=purposes,
-            wallet_alias=wallet_alias,
-            wallet=wallet,
-            specter=app.specter,
-            rand=rand,
-            error=error,
-        )
-    else:
-        return render_template(
-            "wallet/settings/wallet_settings.jinja",
-            purposes=purposes,
-            wallet_alias=wallet_alias,
-            wallet=wallet,
-            specter=app.specter,
-            rand=rand,
-            error=error,
-        )
+    return render_template(
+        "wallet/settings/wallet_settings.jinja",
+        purposes=purposes,
+        wallet_alias=wallet_alias,
+        wallet=wallet,
+        specter=app.specter,
+        rand=rand,
+        error=error,
+    )
 
 
 ################## Wallet util endpoints #######################
@@ -947,12 +976,8 @@ def settings(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/combine/", methods=["POST"])
 @login_required
 def combine(wallet_alias):
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     # only post requests
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while combine: %s" % se)
-        return "SpecterError while combine: %s" % se, 500
     # FIXME: ugly...
     txid = request.form.get("txid")
     psbts = [request.form.get("psbt0").strip(), request.form.get("psbt1").strip()]
@@ -1008,11 +1033,7 @@ def combine(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/broadcast/", methods=["GET", "POST"])
 @login_required
 def broadcast(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-    except SpecterError as se:
-        app.logger.error("SpecterError while broadcast: %s" % se)
-        return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     if request.method == "POST":
         tx = request.form.get("tx")
         res = wallet.rpc.testmempoolaccept([tx])[0]
@@ -1046,6 +1067,20 @@ def decoderawtx(wallet_alias):
             if "blockhash" in tx and "blockheight" not in tx:
                 tx["blockheight"] = wallet.rpc.getblockheader(tx["blockhash"])["height"]
             ##################### Remove until here after dropping Core v0.19 support #####################
+
+            if tx["confirmations"] == 0:
+                tx["is_purged"] = wallet.is_tx_purged(txid)
+                try:
+                    if wallet._transactions[txid].get("category", "") == "receive":
+                        tx["fee"] = (
+                            wallet.rpc.getmempoolentry(txid)["fees"]["modified"] * -1
+                        )
+                except Exception as e:
+                    handle_exception(e)
+                    app.logger.warning(
+                        f"Failed to get fees from mempool entry for transaction: {txid}. Error: {e}"
+                    )
+
             return {
                 "success": True,
                 "tx": tx,
@@ -1087,118 +1122,98 @@ def get_label(wallet_alias):
             "label": label,
         }
     except Exception as e:
-        logging.exception(e)
-        return "Error while get_label: %s" % e, 500
+        handle_exception(e)
+        return {
+            "success": False,
+            "error": f"Exception trying to get address label: Error: {e}",
+        }
 
 
 @wallets_endpoint.route("/wallet/<wallet_alias>/set_label", methods=["POST"])
 @login_required
 def set_label(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-        address = request.form["address"]
-        label = request.form["label"]
-        wallet.setlabel(address, label)
-        return {"success": True}
-    except Exception as e:
-        logging.exception(e)
-        return "Error while set_label: %s" % e, 500
+
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
+    address = request.form["address"]
+    label = request.form["label"]
+    wallet.setlabel(address, label)
+    return {"success": True}
 
 
 @wallets_endpoint.route("/wallet/<wallet_alias>/txlist", methods=["POST"])
 @login_required
 @app.csrf.exempt
 def txlist(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-        idx = int(request.form.get("idx", 0))
-        limit = int(request.form.get("limit", 100))
-        search = request.form.get("search", None)
-        sortby = request.form.get("sortby", None)
-        sortdir = request.form.get("sortdir", "asc")
-        fetch_transactions = request.form.get("fetch_transactions", False)
-        txlist = wallet.txlist(
-            fetch_transactions=fetch_transactions,
-            validate_merkle_proofs=app.specter.config.get(
-                "validate_merkle_proofs", False
-            ),
-            current_blockheight=app.specter.info["blocks"],
-        )
-        return process_txlist(
-            txlist, idx=idx, limit=limit, search=search, sortby=sortby, sortdir=sortdir
-        )
-    except Exception as e:
-        logging.exception(e)
-        return "Error while getting txlist: %s" % e, 500
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
+    idx = int(request.form.get("idx", 0))
+    limit = int(request.form.get("limit", 100))
+    search = request.form.get("search", None)
+    sortby = request.form.get("sortby", None)
+    sortdir = request.form.get("sortdir", "asc")
+    fetch_transactions = request.form.get("fetch_transactions", False)
+    txlist = wallet.txlist(
+        fetch_transactions=fetch_transactions,
+        validate_merkle_proofs=app.specter.config.get("validate_merkle_proofs", False),
+        current_blockheight=app.specter.info["blocks"],
+    )
+    return process_txlist(
+        txlist, idx=idx, limit=limit, search=search, sortby=sortby, sortdir=sortdir
+    )
 
 
 @wallets_endpoint.route("/wallet/<wallet_alias>/utxo_list", methods=["POST"])
 @login_required
 @app.csrf.exempt
 def utxo_list(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-        idx = int(request.form.get("idx", 0))
-        limit = int(request.form.get("limit", 100))
-        search = request.form.get("search", None)
-        sortby = request.form.get("sortby", None)
-        sortdir = request.form.get("sortdir", "asc")
-        txlist = wallet.full_utxo
-        for tx in txlist:
-            if not tx.get("label", None):
-                tx["label"] = wallet.getlabel(tx["address"])
-        return process_txlist(
-            txlist, idx=idx, limit=limit, search=search, sortby=sortby, sortdir=sortdir
-        )
-    except Exception as e:
-        logging.exception(e)
-        return "Error while getting utxo list: %s" % e, 500
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
+    idx = int(request.form.get("idx", 0))
+    limit = int(request.form.get("limit", 100))
+    search = request.form.get("search", None)
+    sortby = request.form.get("sortby", None)
+    sortdir = request.form.get("sortdir", "asc")
+    txlist = wallet.full_utxo
+    for tx in txlist:
+        if not tx.get("label", None):
+            tx["label"] = wallet.getlabel(tx["address"])
+    return process_txlist(
+        txlist, idx=idx, limit=limit, search=search, sortby=sortby, sortdir=sortdir
+    )
 
 
 @wallets_endpoint.route("/wallets_overview/txlist", methods=["POST"])
 @login_required
 @app.csrf.exempt
 def wallets_overview_txlist():
-    try:
-        idx = int(request.form.get("idx", 0))
-        limit = int(request.form.get("limit", 100))
-        search = request.form.get("search", None)
-        sortby = request.form.get("sortby", None)
-        sortdir = request.form.get("sortdir", "asc")
-        fetch_transactions = request.form.get("fetch_transactions", False)
-        txlist = app.specter.wallet_manager.full_txlist(
-            fetch_transactions=fetch_transactions,
-            validate_merkle_proofs=app.specter.config.get(
-                "validate_merkle_proofs", False
-            ),
-            current_blockheight=app.specter.info["blocks"],
-        )
-        return process_txlist(
-            txlist, idx=idx, limit=limit, search=search, sortby=sortby, sortdir=sortdir
-        )
-    except Exception as e:
-        logging.exception(e)
-        return "Error while getting full txlist: %s" % e, 500
+    idx = int(request.form.get("idx", 0))
+    limit = int(request.form.get("limit", 100))
+    search = request.form.get("search", None)
+    sortby = request.form.get("sortby", None)
+    sortdir = request.form.get("sortdir", "asc")
+    fetch_transactions = request.form.get("fetch_transactions", False)
+    txlist = app.specter.wallet_manager.full_txlist(
+        fetch_transactions=fetch_transactions,
+        validate_merkle_proofs=app.specter.config.get("validate_merkle_proofs", False),
+        current_blockheight=app.specter.info["blocks"],
+    )
+    return process_txlist(
+        txlist, idx=idx, limit=limit, search=search, sortby=sortby, sortdir=sortdir
+    )
 
 
 @wallets_endpoint.route("/wallets_overview/utxo_list", methods=["POST"])
 @login_required
 @app.csrf.exempt
 def wallets_overview_utxo_list():
-    try:
-        idx = int(request.form.get("idx", 0))
-        limit = int(request.form.get("limit", 100))
-        search = request.form.get("search", None)
-        sortby = request.form.get("sortby", None)
-        sortdir = request.form.get("sortdir", "asc")
-        fetch_transactions = request.form.get("fetch_transactions", False)
-        txlist = app.specter.wallet_manager.full_utxo()
-        return process_txlist(
-            txlist, idx=idx, limit=limit, search=search, sortby=sortby, sortdir=sortdir
-        )
-    except Exception as e:
-        logging.exception(e)
-        return "Error while getting full utxo list: %s" % e, 500
+    idx = int(request.form.get("idx", 0))
+    limit = int(request.form.get("limit", 100))
+    search = request.form.get("search", None)
+    sortby = request.form.get("sortby", None)
+    sortdir = request.form.get("sortdir", "asc")
+    fetch_transactions = request.form.get("fetch_transactions", False)
+    txlist = app.specter.wallet_manager.full_utxo()
+    return process_txlist(
+        txlist, idx=idx, limit=limit, search=search, sortby=sortby, sortdir=sortdir
+    )
 
 
 @wallets_endpoint.route("/wallet/<wallet_alias>/addresses_list/", methods=["POST"])
@@ -1216,29 +1231,24 @@ def addresses_list(wallet_alias):
                 (index, address, label, used, utxo, amount)
         sortdir: 'asc' (ascending) or 'desc' (descending) order
         addressType: the current tab address type ('receive' or 'change')"""
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
 
-        idx = int(request.form.get("idx", 0))
-        limit = int(request.form.get("limit", 100))
-        sortby = request.form.get("sortby", None)
-        sortdir = request.form.get("sortdir", "asc")
-        address_type = request.form.get("addressType", "receive")
+    idx = int(request.form.get("idx", 0))
+    limit = int(request.form.get("limit", 100))
+    sortby = request.form.get("sortby", None)
+    sortdir = request.form.get("sortdir", "asc")
+    address_type = request.form.get("addressType", "receive")
 
-        addresses_list = wallet.addresses_info(address_type == "change")
+    addresses_list = wallet.addresses_info(address_type == "change")
 
-        result = process_addresses_list(
-            addresses_list, idx=idx, limit=limit, sortby=sortby, sortdir=sortdir
-        )
+    result = process_addresses_list(
+        addresses_list, idx=idx, limit=limit, sortby=sortby, sortdir=sortdir
+    )
 
-        return {
-            "addressesList": json.dumps(result["addressesList"]),
-            "pageCount": result["pageCount"],
-        }
-
-    except Exception as e:
-        logging.exception(e)
-        return "Error while getting addresses_list: %s" % e, 500
+    return {
+        "addressesList": json.dumps(result["addressesList"]),
+        "pageCount": result["pageCount"],
+    }
 
 
 @wallets_endpoint.route("/wallet/<wallet_alias>/addressinfo/", methods=["POST"])
@@ -1256,9 +1266,11 @@ def addressinfo(wallet_alias):
                 "address": address,
                 "descriptor": descriptor,
                 "walletName": wallet.name,
+                "isMine": not address_info.is_external,
                 **address_info,
             }
     except Exception as e:
+
         app.logger.warning("Failed to fetch address data. Exception: {}".format(e))
     return {"success": False}
 
@@ -1330,35 +1342,29 @@ def addresses_list_csv(wallet_alias):
 @wallets_endpoint.route("/wallet/<wallet_alias>/transactions.csv")
 @login_required
 def tx_history_csv(wallet_alias):
-    try:
-        wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
-        validate_merkle_proofs = app.specter.config.get("validate_merkle_proofs", False)
-        txlist = wallet.txlist(validate_merkle_proofs=validate_merkle_proofs)
-        search = request.args.get("search", None)
-        sortby = request.args.get("sortby", "time")
-        sortdir = request.args.get("sortdir", "desc")
-        txlist = json.loads(
-            process_txlist(
-                txlist, idx=0, limit=0, search=search, sortby=sortby, sortdir=sortdir
-            )["txlist"]
-        )
-        includePricesHistory = request.args.get("exportPrices", "false") == "true"
+    wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
+    validate_merkle_proofs = app.specter.config.get("validate_merkle_proofs", False)
+    txlist = wallet.txlist(validate_merkle_proofs=validate_merkle_proofs)
+    search = request.args.get("search", None)
+    sortby = request.args.get("sortby", "time")
+    sortdir = request.args.get("sortdir", "desc")
+    txlist = json.loads(
+        process_txlist(
+            txlist, idx=0, limit=0, search=search, sortby=sortby, sortdir=sortdir
+        )["txlist"]
+    )
+    includePricesHistory = request.args.get("exportPrices", "false") == "true"
 
-        # stream the response as the data is generated
-        response = Response(
-            txlist_to_csv(
-                wallet, txlist, app.specter, current_user, includePricesHistory
-            ),
-            mimetype="text/csv",
-        )
-        # add a filename
-        response.headers.set(
-            "Content-Disposition", "attachment", filename="transactions.csv"
-        )
-        return response
-    except Exception as e:
-        logging.exception(e)
-        return "Failed to export wallet history. Error: %s" % e, 500
+    # stream the response as the data is generated
+    response = Response(
+        txlist_to_csv(wallet, txlist, app.specter, current_user, includePricesHistory),
+        mimetype="text/csv",
+    )
+    # add a filename
+    response.headers.set(
+        "Content-Disposition", "attachment", filename="transactions.csv"
+    )
+    return response
 
 
 # Export wallet UTXO list
