@@ -6,11 +6,16 @@ import traceback
 import random
 import time
 import zipfile
+import platform
+import secrets
 import requests
+import signal
 from io import BytesIO
-from .helpers import migrate_config, deep_update, clean_psbt, is_testnet
+from .helpers import deep_update, clean_psbt, is_testnet
 from .util.checker import Checker
-from .rpc import autodetect_rpc_confs, get_default_datadir, RpcError
+from .rpc import autodetect_rpc_confs, detect_rpc_confs, get_default_datadir, RpcError
+from .bitcoind import BitcoindPlainController
+from .tor_daemon import TorDaemonController
 from urllib3.exceptions import NewConnectionError
 from requests.exceptions import ConnectionError
 from .rpc import BitcoinRPC
@@ -20,7 +25,15 @@ from .user_manager import UserManager
 from .persistence import write_json_file, read_json_file
 from .user import User
 from .util.price_providers import update_price
+from .util.tor import get_tor_daemon_suffix
 import threading
+from urllib.parse import urlparse
+from stem.control import Controller
+from .specter_error import SpecterError, ExtProcTimeoutException
+from sys import exit
+from .util.setup_states import SETUP_STATES
+from .managers.otp_manager import OtpManager
+from .managers.config_manager import ConfigManager
 
 logger = logging.getLogger(__name__)
 
@@ -77,7 +90,7 @@ def get_rpc(
 
 
 class Specter:
-    """ A central Object mostly holding app-settings """
+    """A central Object mostly holding app-settings"""
 
     # use this lock for all fs operations
     lock = threading.Lock()
@@ -93,56 +106,91 @@ class Specter:
 
         self.data_folder = data_folder
 
+        # the rpc-object. Currently we only have one. If we have Node-Managers, we would need
+        # either many of them and register them with a keyword or something like that
         self.rpc = None
-        self.user_manager = UserManager(self)
 
-        self.file_config = None  # what comes from config file
-        self.arg_config = config  # what comes from arguments
-
-        # wallet that is currently rescnning with utxorescan
+        # wallet that is currently rescanning with utxorescan
         # can be only one at a time
         self.utxorescanwallet = None
 
-        # default config
-        self.config = {
-            "rpc": {
-                "autodetect": True,
-                "datadir": get_default_datadir(),
-                "user": "",
-                "password": "",
-                "port": "",
-                "host": "localhost",  # localhost
-                "protocol": "http",  # https for the future
-            },
-            "auth": {
-                "method": "none",
-                "password_min_chars": 6,
-                "rate_limit": 10,
-                "registration_link_timeout": 1,
-            },
-            "explorers": {"main": "", "test": "", "regtest": "", "signet": ""},
-            "proxy_url": "socks5h://localhost:9050",  # Tor proxy URL
-            "only_tor": False,
-            "tor_control_port": "",
-            "tor_status": False,
-            "hwi_bridge_url": "/hwi/api/",
-            # unique id that will be used in wallets path in Bitcoin Core
-            # empty by default for backward-compatibility
-            "uid": "",
-            "unit": "btc",
-            "price_check": False,
-            "alt_rate": 1,
-            "alt_symbol": "BTC",
-            "price_provider": "",
-            "validate_merkle_proofs": False,
-        }
+        self.user_manager = UserManager(self)
 
+        self._config_manager = ConfigManager(self.data_folder, config)
+
+        self.torbrowser_path = os.path.join(
+            self.data_folder, f"tor-binaries/tor{get_tor_daemon_suffix()}"
+        )
+
+        self.bitcoind_path = os.path.join(
+            self.data_folder, "bitcoin-binaries/bin/bitcoind"
+        )
+
+        if platform.system() == "Windows":
+            self.bitcoind_path += ".exe"
+
+        self._bitcoind = None
+        self._tor_daemon = None
+
+        self.node_status = None
+        self.setup_status = {
+            "stage": "start",
+            "bitcoind": {
+                "stage_progress": -1,
+                "stage": "",
+                "error": "",
+            },
+            "torbrowser": {
+                "stage_progress": -1,
+                "stage": "",
+                "error": "",
+            },
+        }
         # health check: loads config, tests rpc
         # also loads and checks wallets for all users
         try:
             self.check(check_all=True)
+            rpc_conf = next(
+                (
+                    conf
+                    for conf in detect_rpc_confs(
+                        datadir=os.path.expanduser(
+                            self.config["rpc"]["datadir"]
+                            if self.config["rpc"].get("external_node", True)
+                            else self.config["internal_node"]["datadir"]
+                        )
+                    )
+                    if conf["port"] == 8332
+                ),
+                None,
+            )
+
+            if os.path.isfile(self.torbrowser_path):
+                self.tor_daemon.start_tor_daemon()
         except Exception as e:
             logger.error(e)
+
+        if not self.config_manager.data["rpc"].get("external_node", True):
+            try:
+                self.bitcoind.start_bitcoind(
+                    datadir=os.path.expanduser(self.config["internal_node"]["datadir"]),
+                    timeout=15,  # At the initial startup, we don't wait on bitcoind
+                )
+            except ExtProcTimeoutException as e:
+                logger.error(e)
+                e.check_logfile(
+                    os.path.join(self.config["internal_node"]["datadir"], "debug.log")
+                )
+                logger.error(e.get_logger_friendly())
+            except SpecterError as e:
+                logger.error(e)
+                # Likely files of bitcoind were not found. Maybe deleted by the user?
+            finally:
+                try:
+                    self.set_bitcoind_pid(self.bitcoind.bitcoind_proc.pid)
+                except Exception as e:
+                    logger.error(e)
+        self.update_tor_controller()
         self.checker = Checker(lambda: self.check(check_all=True), desc="health")
         self.checker.start()
         self.price_checker = Checker(
@@ -151,6 +199,24 @@ class Specter:
         if self.price_check and self.price_provider:
             self.price_checker.start()
 
+        # This is for CTRL-C --> SIGINT
+        signal.signal(signal.SIGINT, self.cleanup_on_exit)
+        # This is for kill $pid --> SIGTERM
+        signal.signal(signal.SIGTERM, self.cleanup_on_exit)
+
+    def cleanup_on_exit(self, signum=0, frame=0):
+        if self._tor_daemon:
+            logger.info("Specter exit cleanup: Stopping Tor daemon")
+            self._tor_daemon.stop_tor_daemon()
+
+        if self._bitcoind:
+            logger.info("Specter exit cleanup: Stopping bitcoind")
+            self._bitcoind.stop_bitcoind()
+
+        logger.info("Closing Specter after cleanup")
+        # For some reason we need to explicitely exit here. Otherwise it will hang
+        exit(0)
+
     def check(self, user=None, check_all=False):
         """
         Checks and updates everything for a particular user:
@@ -167,15 +233,17 @@ class Specter:
         rpc = self.rpc
         if rpc is None or not rpc.test_connection():
             rpc = get_rpc(
-                self.config["rpc"],
+                self.config_manager.rpc_conf,
                 self.rpc,
                 proxy_url=self.proxy_url,
                 only_tor=self.only_tor,
             )
 
+        self.check_node_info()
+
         # if rpc is not available
         # do checks more often, once in 20 seconds
-        if rpc is None:
+        if rpc is None or self.info.get("initialblockdownload", True):
             period = 20
         else:
             period = 600
@@ -183,14 +251,18 @@ class Specter:
             self.checker.period = period
         self.rpc = rpc
 
-        self.check_node_info()
         if not check_all:
             # find proper user
             user = self.user_manager.get_user(user)
-            self.check_for_user(user)
+            user.check()
         else:
             for u in self.user_manager.users:
-                self.check_for_user(u)
+                u.check()
+
+    @property
+    def config(self):
+        """A convenience property simply redirecting to the config_manager"""
+        return self.config_manager.data
 
     def check_node_info(self):
         self._is_configured = self.rpc is not None
@@ -254,42 +326,6 @@ class Specter:
             return "_" + user.id
         return ""
 
-    def check_wallet_manager(self, user=None):
-        """Updates wallet manager for a particular user"""
-        user = self.user_manager.get_user(user)
-        wallets_rpcpath = "specter%s" % self.config["uid"]
-        wallets_folder = os.path.join(self.data_folder, f"wallets{user.folder_id}")
-        # if chain, user or data folder changed
-        wallet_manager = user.wallet_manager
-        if (
-            wallet_manager is None
-            or wallet_manager.data_folder != self.data_folder
-            or wallet_manager.rpc_path != wallets_rpcpath
-            or wallet_manager.chain != self.chain
-        ):
-            wallet_manager = WalletManager(
-                self.bitcoin_core_version_raw,
-                wallets_folder,
-                self.rpc,
-                self.chain,
-                user.device_manager,
-                path=wallets_rpcpath,
-            )
-            user.wallet_manager = wallet_manager
-        else:
-            wallet_manager.update(wallets_folder, self.rpc, chain=self.chain)
-
-    def check_device_manager(self, user=None):
-        """Updates device manager for a particular user"""
-        user = self.user_manager.get_user(user)
-        devices_folder = os.path.join(self.data_folder, f"devices{user.folder_id}")
-        device_manager = user.device_manager
-        if device_manager is None:
-            device_manager = DeviceManager(devices_folder)
-            user.device_manager = device_manager
-        else:
-            device_manager.update(data_folder=devices_folder)
-
     def check_config(self):
         """
         Updates config if file config have changed.
@@ -298,40 +334,7 @@ class Specter:
         - file config from config.json
         - arg_config passed in constructor
         """
-
-        # if config.json file exists - load from it
-        if os.path.isfile(self.config_fname):
-            with self.lock:
-                self.file_config = read_json_file(self.config_fname)
-                migrate_config(self.file_config)
-                deep_update(self.config, self.file_config)
-            # otherwise - create one and assign unique id
-        else:
-            # unique id of specter
-            if self.config["uid"] == "":
-                self.config["uid"] = (
-                    random.randint(0, 256 ** 8).to_bytes(8, "big").hex()
-                )
-            self._save()
-
-        # config from constructor overrides file config
-        deep_update(self.config, self.arg_config)
-
-    def check_for_user(self, user=None):
-        """
-        Performs device and wallet manager check for particular user
-        """
-        user = self.user_manager.get_user(user)
-        self.check_device_manager(user)
-        self.check_wallet_manager(user)
-
-    def add_user(self, user):
-        if user in self.user_manager.users:
-            return
-        user.wallet_manager = None
-        user.device_manager = None
-        self.user_manager.add_user(user)
-        self.check_for_user(user)
+        self.config_manager.check_config()
 
     def delete_user(self, user):
         if user not in self.user_manager.users:
@@ -341,11 +344,10 @@ class Specter:
         user.device_manager.delete(self)
         self.user_manager.delete_user(user)
 
+    # mark
     @property
     def bitcoin_datadir(self):
-        if "datadir" in self.config["rpc"]:
-            return os.path.expanduser(self.config["rpc"]["datadir"])
-        return get_default_datadir()
+        return self.config_manager.bitcoin_datadir
 
     def abortrescanutxo(self):
         self.rpc.scantxoutset("abort", [])
@@ -355,7 +357,7 @@ class Specter:
         self.utxorescanwallet = None
 
     def test_rpc(self, **kwargs):
-        conf = copy.deepcopy(self.config["rpc"])
+        conf = copy.deepcopy(self.config_manager.data["rpc"])
         conf.update(kwargs)
 
         rpc = get_rpc(
@@ -419,6 +421,7 @@ class Specter:
                 r["code"] = -1
         return r
 
+    # mark
     def _save(self):
         write_json_file(self.config, self.config_fname, lock=self.lock)
 
@@ -426,15 +429,12 @@ class Specter:
     def config_fname(self):
         return os.path.join(self.data_folder, "config.json")
 
+    # mark
     def update_rpc(self, **kwargs):
-        need_update = False
-        for k in kwargs:
-            if self.config["rpc"][k] != kwargs[k]:
-                self.config["rpc"][k] = kwargs[k]
-                need_update = True
+        need_update = self.config_manager.update_rpc(**kwargs)
         if need_update:
             self.rpc = get_rpc(
-                self.config["rpc"],
+                self.config_manager.rpc_conf,
                 None,
                 proxy_url=self.proxy_url,
                 only_tor=self.only_tor,
@@ -443,163 +443,193 @@ class Specter:
             self.check(check_all=True)
         return self.rpc is not None
 
+    # mark
+    def set_bitcoind_pid(self, pid):
+        """set the control pid of the bitcoind daemon"""
+        self.config_manager.set_bitcoind_pid(pid)
+
+    def update_setup_status(self, software_name, stage):
+        self.setup_status[software_name]["error"] = ""
+        if stage in SETUP_STATES:
+            self.setup_status[software_name]["stage"] = SETUP_STATES[stage].get(
+                software_name, stage
+            )
+        else:
+            self.setup_status[software_name]["stage"] = stage
+        self.setup_status[software_name]["stage_progress"] = 0
+
+    def update_setup_download_progress(self, software_name, progress):
+        self.setup_status[software_name]["error"] = ""
+        self.setup_status[software_name]["stage_progress"] = progress
+
+    def update_setup_error(self, software_name, error):
+        self.setup_status[software_name]["error"] = error
+        self.setup_status[software_name]["stage_progress"] = -1
+
+    def reset_setup(self, software_name):
+        self.setup_status[software_name]["error"] = ""
+        self.setup_status[software_name]["stage"] = ""
+        self.setup_status[software_name]["stage_progress"] = -1
+
+    def get_setup_status(self, software_name):
+        if software_name == "bitcoind":
+            installed = os.path.isfile(self.bitcoind_path)
+        elif software_name == "torbrowser":
+            installed = os.path.isfile(self.torbrowser_path)
+        else:
+            installed = False
+
+        return {"installed": installed, **self.setup_status[software_name]}
+
+    # mark
+    def update_use_external_node(self, use_external_node):
+        """set whatever specter should connect to internal or external node"""
+        self.config_manager.update_use_external_node(use_external_node)
+
+    # mark
     def update_auth(self, method, rate_limit, registration_link_timeout):
-        """ simply persisting the current auth-choice """
-        auth = self.config["auth"]
-        if auth["method"] != method:
-            auth["method"] = method
-        if auth["rate_limit"] != rate_limit:
-            auth["rate_limit"] = rate_limit
-        if auth["registration_link_timeout"] != registration_link_timeout:
-            auth["registration_link_timeout"] = registration_link_timeout
-        self._save()
+        """simply persisting the current auth-choice"""
+        self.config_manager.update_auth(method, rate_limit, registration_link_timeout)
 
-    def update_explorer(self, explorer, user):
-        """ update the block explorers urls """
-        user = self.user_manager.get_user(user)
-        # we don't know what chain to change
-        if not self.chain:
-            return
-        if explorer and not explorer.endswith("/"):
-            # make sure the urls end with a "/"
-            explorer += "/"
-        # update the urls in the app config
-        if user.id == "admin":
-            if self.config["explorers"][self.chain] != explorer:
-                self.config["explorers"][self.chain] = explorer
-            self._save()
-        else:
-            user.set_explorer(self, explorer)
+    # mark
+    def update_explorer(self, explorer_id, explorer_data, user):
+        """update the block explorers urls"""
+        self.config_manager.update_explorer(
+            explorer_id, explorer_data, user, self.chain
+        )
 
+    # mark
+    def update_fee_estimator(self, fee_estimator, custom_url, user):
+        """update the fee estimator option and its url if custom"""
+        self.config_manager.update_fee_estimator(fee_estimator, custom_url, user)
+
+    # mark
     def update_proxy_url(self, proxy_url, user):
-        """ update the Tor proxy url """
-        if self.config["proxy_url"] != proxy_url:
-            self.config["proxy_url"] = proxy_url
-            self._save()
+        """update the Tor proxy url"""
+        self.config_manager.update_proxy_url(proxy_url, user)
 
+    # mark
     def toggle_tor_status(self):
-        """ toggle the Tor status """
-        self.config["tor_status"] = not self.config["tor_status"]
-        self._save()
+        """toggle the Tor status"""
+        self.config_manager.toggle_tor_status()
 
+    # mark
     def update_only_tor(self, only_tor, user):
-        """ switch whatever to use Tor for all calls """
-        if self.config["only_tor"] != only_tor:
-            self.config["only_tor"] = only_tor
-            self._save()
+        """switch whatever to use Tor for all calls"""
+        self.config_manager.update_only_tor(only_tor, user)
 
+    # mark
     def update_tor_control_port(self, tor_control_port, user):
-        """ set the control port of the tor daemon """
-        if self.config["tor_control_port"] != tor_control_port:
-            self.config["tor_control_port"] = tor_control_port
-            self._save()
+        """set the control port of the tor daemon"""
+        if self.config_manager.update_tor_control_port:
+            self.update_tor_controller()
 
+    # mark
+    def generate_torrc_password(self, overwrite=False):
+        self.config_manager.generate_torrc_password(overwrite)
+
+    def update_tor_controller(self):
+        if "torrc_password" not in self.config:
+            # Will be missing if the user did not go through the built-in Tor setup
+            self.generate_torrc_password()
+        try:
+            tor_control_address = urlparse(self.proxy_url).netloc.split(":")[0]
+            if tor_control_address == "localhost":
+                tor_control_address = "127.0.0.1"
+            self._tor_controller = Controller.from_port(
+                address=tor_control_address,
+                port=int(self.tor_control_port) if self.tor_control_port else "default",
+            )
+            self._tor_controller.authenticate(
+                password=self.config.get("torrc_password", "")
+            )
+        except Exception as e:
+            logger.warning(f"Failed to connect to Tor control port. Error: {e}")
+            self._tor_controller = None
+
+    @property
+    def tor_daemon(self):
+        if os.path.isfile(self.torbrowser_path) and os.path.join(
+            self.data_folder, "torrc"
+        ):
+            if not self._tor_daemon:
+                self._tor_daemon = TorDaemonController(
+                    tor_daemon_path=self.torbrowser_path,
+                    tor_config_path=os.path.join(self.data_folder, "torrc"),
+                )
+            return self._tor_daemon
+        raise SpecterError(
+            "Tor daemon files missing. Make sure Tor is installed within Specter"
+        )
+
+    @property
+    def bitcoind(self):
+        if os.path.isfile(self.bitcoind_path):
+            if not self._bitcoind:
+                self._bitcoind = BitcoindPlainController(
+                    bitcoind_path=self.bitcoind_path,
+                    rpcport=8332,
+                    network="mainnet",
+                    rpcuser=self.config["internal_node"]["user"],
+                    rpcpassword=self.config["internal_node"]["password"],
+                )
+            return self._bitcoind
+        raise SpecterError(
+            "Bitcoin Core files missing. Make sure Bitcoin Core is installed within Specter"
+        )
+
+    def is_tor_dameon_running(self):
+        return self._tor_daemon and self._tor_daemon.is_running()
+
+    def is_bitcoind_running(self):
+        return self._bitcoind and self._bitcoind.check_existing()
+
+    @property
+    def tor_controller(self):
+        if self._tor_controller:
+            return self._tor_controller
+        self.update_tor_controller()
+        if self._tor_controller:
+            return self._tor_controller
+        raise SpecterError(
+            "Failed to connect to the Tor daemon. Make sure ControlPort is properly configured."
+        )
+
+    # mark
     def update_hwi_bridge_url(self, url, user):
-        """ update the hwi bridge url to use """
-        user = self.user_manager.get_user(user)
-        if url and not url.endswith("/"):
-            # make sure the urls end with a "/"
-            url += "/"
-        # a few dummy checks:
-        # no schema and not local
-        if "://" not in url and not url.startswith("/"):
-            url = "http://" + url
-        # wrong ending:
-        if url.endswith("/hwi/settings/"):
-            url = url.replace("/hwi/settings/", "/hwi/api/")
-        # no ending
-        if not url.endswith("/hwi/api/"):
-            url += "hwi/api/"
-
-        if user.is_admin:
-            self.config["hwi_bridge_url"] = url
-            self._save()
-        else:
-            user.set_hwi_bridge_url(self, url)
+        """update the hwi bridge url to use"""
+        self.config_manager.update_hwi_bridge_url(url, user)
 
+    # mark
     def update_unit(self, unit, user):
-        if user.is_admin:
-            self.config["unit"] = unit
-            self._save()
-        else:
-            user.set_unit(self, unit)
+        self.config_manager.update_unit(unit, user)
 
+    # mark
     def update_price_check_setting(self, price_check_bool, user):
-        if user.is_admin:
-            self.config["price_check"] = price_check_bool
-            self._save()
-        else:
-            user.set_price_check(self, price_check_bool)
-        if price_check_bool and (self.price_provider and self.user == user):
-            self.price_checker.start()
-        else:
-            self.price_checker.stop()
+        self.config_manager.update_price_check_setting(price_check_bool, user)
 
+    # mark
     def update_price_provider(self, price_provider, user):
-        if user.is_admin:
-            self.config["price_provider"] = price_provider
-            self._save()
-        else:
-            user.set_price_provider(self, price_provider)
+        self.config_manager.update_price_provider(price_provider, user)
 
+    # mark needs User-Type injection
+    def update_weight_unit(self, weight_unit, user):
+        self.config_manager.update_weight_unit(weight_unit, user)
+
+    # mark needs User-Type injection
     def update_alt_rate(self, alt_rate, user):
-        alt_rate = round(float(alt_rate), 2)
-        if user.is_admin:
-            self.config["alt_rate"] = alt_rate
-            self._save()
-        else:
-            user.set_alt_rate(self, alt_rate)
+        self.config_manager.update_alt_rate(alt_rate, user)
 
+    # mark
     def update_alt_symbol(self, alt_symbol, user):
-        if user.is_admin:
-            self.config["alt_symbol"] = alt_symbol
-            self._save()
-        else:
-            user.set_alt_symbol(self, alt_symbol)
+        self.config_manager.update_alt_symbol(alt_symbol, user)
 
+    # mark logic!
     def update_merkleproof_settings(self, validate_bool):
         if validate_bool is True and self.info.get("pruned") is True:
             validate_bool = False
             logger.warning("Cannot enable merkleproof setting on pruned node.")
-
-        self.config["validate_merkle_proofs"] = validate_bool
-        self._save()
-
-    def add_new_user_otp(self, otp_dict):
-        """ adds an OTP for user registration """
-        if "new_user_otps" not in self.config:
-            self.config["new_user_otps"] = []
-        self.config["new_user_otps"].append(otp_dict)
-        self._save()
-
-    def validate_new_user_otp(self, otp):
-        """ validates an OTP for user registration and removes it if expired"""
-        if "new_user_otps" not in self.config:
-            return False
-        now = time.time()
-        for i, otp_dict in enumerate(self.config["new_user_otps"]):
-            if otp_dict["otp"] == otp:
-                if (
-                    "expiry" in otp_dict
-                    and otp_dict["expiry"] < now
-                    and otp_dict["expiry"] > 0
-                ):
-                    del self.config["new_user_otps"][i]
-                    self._save()
-                    return False
-                return True
-        return False
-
-    def remove_new_user_otp(self, otp):
-        """ removes an OTP for user registration"""
-        if "new_user_otps" not in self.config:
-            return False
-        for i, otp_dict in enumerate(self.config["new_user_otps"]):
-            if otp_dict["otp"] == otp:
-                del self.config["new_user_otps"][i]
-                self._save()
-                return True
-        return False
+        self.config_manager.update_merkleproof_settings(validate_bool)
 
     def combine(self, psbt_arr):
         # backward compatibility with current Core psbt parser
@@ -618,23 +648,6 @@ class Specter:
     def estimatesmartfee(self, blocks):
         return self.rpc.estimatesmartfee(blocks)
 
-    def get_default_explorer(self):
-        """
-        Returns a blockexplorer url:
-        user-defined if it's set, otherwise
-        blockstream.info for main and testnet,
-        bc-2.jp for signet
-        """
-        # not None or ""
-        if self.explorer:
-            return self.explorer
-        if self.chain == "main":
-            return "https://blockstream.info/"
-        elif self.chain == "test":
-            return "https://blockstream.info/testnet/"
-        elif self.chain == "signet":
-            return "https://explorer.bc-2.jp/"
-
     @property
     def is_running(self):
         return self._is_running
@@ -675,6 +688,14 @@ class Specter:
     def explorer(self):
         return self.user_config.get("explorers", {}).get(self.chain, "")
 
+    @property
+    def explorer_id(self):
+        return self.user_config.get("explorer_id", {}).get(self.chain, "CUSTOM")
+
+    @property
+    def fee_estimator(self):
+        return self.user_config.get("fee_estimator", "mempool")
+
     @property
     def proxy_url(self):
         return self.user_config.get("proxy_url", "socks5h://localhost:9050")
@@ -703,6 +724,10 @@ class Specter:
     def price_provider(self):
         return self.user_config.get("price_provider", False)
 
+    @property
+    def weight_unit(self):
+        return self.user_config.get("weight_unit", "oz")
+
     @property
     def alt_rate(self):
         return self.user_config.get("alt_rate", 1)
@@ -713,7 +738,7 @@ class Specter:
 
     @property
     def admin(self):
-        for u in self.users:
+        for u in self.user_manager.users:
             if u.is_admin:
                 return u
 
@@ -721,6 +746,12 @@ class Specter:
     def user(self):
         return self.user_manager.user
 
+    @property
+    def config_manager(self):
+        if not hasattr(self, "_config_manager"):
+            self._config_manager = ConfigManager(self.data_folder)
+        return self._config_manager
+
     @property
     def device_manager(self):
         return self.user.device_manager
@@ -729,6 +760,12 @@ class Specter:
     def wallet_manager(self):
         return self.user.wallet_manager
 
+    @property
+    def otp_manager(self):
+        if not hasattr(self, "_otp_manager"):
+            self._otp_manager = OtpManager(self.data_folder)
+        return self._otp_manager
+
     def requests_session(self, force_tor=False):
         requests_session = requests.Session()
         if self.only_tor or force_tor:
@@ -758,3 +795,14 @@ class Specter:
                     )
         memory_file.seek(0)
         return memory_file
+
+
+class SpecterConfiguration:
+    """An abstract class which only holds functionality relevant for storage of information mostly
+    deferring to ConfigManager.
+    Do not put logic in here, which is not directly relevant for the config.json.
+    Do not deal with the config.json directly but implement that in the ConfigManager
+    """
+
+    pass
+    # ToDo: move all the methods above here.
