@@ -5,12 +5,17 @@ class HWIBridge {
         this.chain = chain;
         this.in_progress = false;
     }
-    async fetch(command, params={}){
+    async fetch(command, params={}, timeout=0){
         if(this.in_progress){
             throw "HWI is busy processing previous request.";
         }
         this.in_progress = true;
         let data = null;
+        const controller = new AbortController()
+
+        if (timeout > 0) {
+            const timeoutId = setTimeout(() => controller.abort(), timeout);
+        }
         try{
             if (command != 'detect_device' && command != 'enumerate') {
                 params.chain = this.chain;
@@ -21,12 +26,13 @@ class HWIBridge {
                     'Accept': 'application/json',
                     'Content-Type': 'application/json'
                 },
+                signal: controller.signal,
                 body: JSON.stringify({
                     'jsonrpc': '2.0', 
                     'method': command, 
                     'id': 1,
                     params,
-                    forwarded_request: (this.url !== '/hwi/api/')
+                    forwarded_request: (this.url !== '/hwi/api/'),
                 })
             }).then(response => response.json());
         }finally{
@@ -37,10 +43,10 @@ class HWIBridge {
         }
         return data.result;
     }
-    async enumerate(passphrase=""){
+    async enumerate(passphrase="", useTimeout){
         return await this.fetch("enumerate", { 
             passphrase
-        });
+        }, (useTimeout ? 60000 : 0));
     }
     async detectDevice(type, rescan=true){
         // TODO: fingerprint, path, type
@@ -59,6 +65,14 @@ class HWIBridge {
         });
     }
 
+    async getBitbox02PairingCode() {
+        /**
+            Asks the HWI server for a pairing code for BitBox02.
+            Returns {"code": ""} with the code or an empty string if none found.
+        **/
+        return await this.fetch('bitbox02_pairing', {});
+    }
+
     async promptPin(device, passphrase="") {
         /**
             Tells the server to send the 'promptpin' command to the device.
