@@ -4,12 +4,13 @@ from .device import Device
 from .key import Key
 from .util.merkleblock import is_valid_merkle_proof
 from .helpers import der_to_bytes
-from embit import base58
+from embit import base58, bip32
 from .util.descriptor import Descriptor, sort_descriptor, AddChecksum
 from .util.xpub import get_xpub_fingerprint
 from .util.tx import decoderawtransaction
 from .persistence import write_json_file, delete_file
-from hwilib.serializations import PSBT, CTransaction
+from hwilib.tx import CTransaction
+from hwilib.psbt import PSBT, KeyOriginInfo
 from io import BytesIO
 from .specter_error import SpecterError
 import threading
@@ -133,12 +134,32 @@ class Wallet:
         """Load transactions from Bitcoin Core"""
         arr = []
         idx = 0
+        # unconfirmed_selftransfers needed since Bitcoin Core does not properly list `selftransfer` txs in `listtransactions` command
+        # Until v0.21, it listed there consolidations to a receive address, but not change address
+        # Since v0.21, it does not list there consolidations at all
+        # Therefore we need to check here if a transaction might got confirmed
+        # NOTE: This might be a problem in case of re-org...
+        # More details: https://github.com/cryptoadvance/specter-desktop/issues/996
+        unconfirmed_selftransfers = [
+            txid
+            for txid in self._transactions
+            if self._transactions[txid].get("category", "") == "selftransfer"
+            and not self._transactions[txid].get("blockhash", None)
+        ]
+        unconfirmed_selftransfers_txs = []
+        if unconfirmed_selftransfers:
+            unconfirmed_selftransfers_txs = self.rpc.multi(
+                [("gettransaction", txid) for txid in unconfirmed_selftransfers]
+            )
         while True:
-            res = self.rpc.listtransactions(
-                "*",
-                LISTTRANSACTIONS_BATCH_SIZE,
-                LISTTRANSACTIONS_BATCH_SIZE * idx,
-                True,
+            res = (
+                self.rpc.listtransactions(
+                    "*",
+                    LISTTRANSACTIONS_BATCH_SIZE,
+                    LISTTRANSACTIONS_BATCH_SIZE * idx,
+                    True,
+                )
+                + [tx["result"] for tx in unconfirmed_selftransfers_txs]
             )
             res = [
                 tx
@@ -258,7 +279,7 @@ class Wallet:
                 obj = self.rpc.listsinceblock()
         txs = obj["transactions"]
         last_block = obj["lastblock"]
-        addresses = [tx["address"] for tx in txs]
+        addresses = [tx["address"] for tx in txs if "address" in tx]
         # remove duplicates
         addresses = list(dict.fromkeys(addresses))
         max_recv = self.address_index - 1
@@ -403,8 +424,8 @@ class Wallet:
     def get_info(self):
         try:
             self.info = self.rpc.getwalletinfo()
-        except Exception:
-            self.info = {}
+        except Exception as e:
+            raise SpecterError(e)
         return self.info
 
     def check_utxo(self):
@@ -439,8 +460,8 @@ class Wallet:
                     pass
             self.full_utxo = sorted(utxo, key=lambda utxo: utxo["time"], reverse=True)
         except Exception as e:
-            logger.error(f"Failed to load utxos, {e}")
             self.full_utxo = []
+            raise SpecterError(f"Failed to load utxos, {e}")
 
     def getdata(self):
         self.fetch_transactions()
@@ -520,6 +541,10 @@ class Wallet:
     def is_multisig(self):
         return len(self.keys) > 1
 
+    @property
+    def keys_count(self):
+        return len(self.keys)
+
     @property
     def locked_amount(self):
         amount = 0
@@ -713,6 +738,30 @@ class Wallet:
         except Exception as e:
             logger.warning("Could not get transaction {}, error: {}".format(txid, e))
 
+    def is_tx_purged(self, txid):
+        # Is tx unconfirmed and no longer in the mempool?
+        try:
+            tx = self.rpc.gettransaction(txid)
+
+            # Do this quick test first to avoid the costlier rpc call
+            if tx["confirmations"] > 0:
+                return False
+
+            return txid not in self.rpc.getrawmempool()
+        except Exception as e:
+            logger.warning("Could not check is_tx_purged {}, error: {}".format(txid, e))
+
+    def abandontransaction(self, txid):
+        # Sanity checks: tx must be unconfirmed and cannot be in the mempool
+        tx = self.rpc.gettransaction(txid)
+        if tx["confirmations"] != 0:
+            raise SpecterError("Cannot abandon a transaction that has a confirmation.")
+        elif txid in self.rpc.getrawmempool():
+            raise SpecterError(
+                "Cannot abandon a transaction that is still in the mempool."
+            )
+        self.rpc.abandontransaction(txid)
+
     def rescanutxo(self, explorer=None, requests_session=None, only_tor=False):
         delete_file(self._transactions.path)
         self.fetch_transactions()
@@ -962,8 +1011,12 @@ class Wallet:
         except:
             return None
 
+    def is_address_mine(self, address):
+        addrinfo = self.get_address_info(address)
+        return addrinfo and not addrinfo.is_external
+
     def get_electrum_file(self):
-        """ Exports the wallet data as Electrum JSON format """
+        """Exports the wallet data as Electrum JSON format"""
         electrum_devices = [
             "bitbox02",
             "coldcard",
@@ -1210,6 +1263,7 @@ class Wallet:
         readonly=False,
         rbf=True,
         existing_psbt=None,
+        rbf_edit_mode=False,
     ):
         """
         fee_rate: in sat/B or BTC/kB. If set to 0 Bitcoin Core sets feeRate automatically.
@@ -1221,17 +1275,20 @@ class Wallet:
         extra_inputs = []
 
         if not existing_psbt:
-            if self.full_available_balance < sum(amounts):
-                raise SpecterError(
-                    "The wallet does not have sufficient funds to make the transaction."
-                )
+            if not rbf_edit_mode:
+                if self.full_available_balance < sum(amounts):
+                    raise SpecterError(
+                        "The wallet does not have sufficient funds to make the transaction."
+                    )
 
             if selected_coins != []:
                 still_needed = sum(amounts)
                 for coin in selected_coins:
                     coin_txid = coin.split(",")[0]
                     coin_vout = int(coin.split(",")[1])
-                    coin_amount = float(coin.split(",")[2])
+                    coin_amount = self.gettransaction(coin_txid, decode=True)["vout"][
+                        coin_vout
+                    ]["value"]
                     extra_inputs.append({"txid": coin_txid, "vout": coin_vout})
                     still_needed -= coin_amount
                     if still_needed < 0:
@@ -1332,7 +1389,63 @@ class Wallet:
 
         return psbt
 
-    def send_rbf_tx(self, txid, fee_rate):
+    def get_rbf_utxo(self, rbf_tx_id):
+        decoded_tx = self.decode_tx(rbf_tx_id)
+        selected_coins = [
+            f"{utxo['txid']}, {utxo['vout']}" for utxo in decoded_tx["used_utxo"]
+        ]
+        rbf_utxo = [
+            {
+                "txid": tx["txid"],
+                "vout": tx["vout"],
+                "details": self.gettransaction(tx["txid"], decode=True)["vout"][
+                    tx["vout"]
+                ],
+            }
+            for tx in decoded_tx["used_utxo"]
+        ]
+        return [
+            {
+                "txid": utxo["txid"],
+                "vout": utxo["vout"],
+                "amount": utxo["details"]["value"],
+                "address": utxo["details"]["addresses"][0],
+                "label": self.getlabel(utxo["details"]["addresses"][0]),
+            }
+            for utxo in rbf_utxo
+        ]
+
+    def decode_tx(self, txid):
+        raw_tx = self.gettransaction(txid)["hex"]
+        raw_psbt = self.rpc.utxoupdatepsbt(
+            self.rpc.converttopsbt(raw_tx, True),
+            [self.recv_descriptor, self.change_descriptor],
+        )
+
+        psbt = self.rpc.decodepsbt(raw_psbt)
+        return {
+            "addresses": [
+                vout["scriptPubKey"]["addresses"][0]
+                for i, vout in enumerate(psbt["tx"]["vout"])
+                if not self.get_address_info(vout["scriptPubKey"]["addresses"][0])
+                or not self.get_address_info(
+                    vout["scriptPubKey"]["addresses"][0]
+                ).change
+            ],
+            "amounts": [
+                vout["value"]
+                for i, vout in enumerate(psbt["tx"]["vout"])
+                if not self.get_address_info(vout["scriptPubKey"]["addresses"][0])
+                or not self.get_address_info(
+                    vout["scriptPubKey"]["addresses"][0]
+                ).change
+            ],
+            "used_utxo": [
+                {"txid": vin["txid"], "vout": vin["vout"]} for vin in psbt["tx"]["vin"]
+            ],
+        }
+
+    def bumpfee(self, txid, fee_rate):
         raw_tx = self.gettransaction(txid)["hex"]
         raw_psbt = self.rpc.utxoupdatepsbt(
             self.rpc.converttopsbt(raw_tx, True),
@@ -1399,17 +1512,18 @@ class Wallet:
             # for multisig add xpub fields
             if len(self.keys) > 1:
                 for k in self.keys:
-                    key = b"\x01" + base58.decode_check(k.xpub)
+                    key = base58.decode_check(k.xpub)
                     if k.fingerprint != "":
                         fingerprint = bytes.fromhex(k.fingerprint)
                     else:
                         fingerprint = get_xpub_fingerprint(k.xpub)
                     if k.derivation != "":
-                        der = der_to_bytes(k.derivation)
+                        der = bip32.parse_path(k.derivation)
                     else:
-                        der = b""
-                    value = fingerprint + der
-                    psbt.unknown[key] = value
+                        der = []
+                    psbt.xpub[key] = KeyOriginInfo(fingerprint, der)
+        else:
+            psbt.xpub = {}
         return psbt.serialize()
 
     def get_signed_devices(self, decodedpsbt):
